import { printBlockString as e } from "./blockString.mjs";

import { printString as n } from "./printString.mjs";

function print(r) {
  if (Array.isArray(r)) {
    return r.map(print);
  } else if (null == r || "object" != typeof r) {
    return r ? "" + r : "";
  }
  switch (r.kind) {
   case "OperationDefinition":
    var t = join([ r.operation, print(r.name) + wrap("(", join(print(r.variableDefinitions), ", "), ")"), join(print(r.directives), " ") ], " ");
    return ("query" === t ? "" : t + " ") + print(r.selectionSet);

   case "VariableDefinition":
    return print(r.variable) + ": " + print(r.type) + wrap(" = ", print(r.defaultValue)) + wrap(" ", join(print(r.directives), " "));

   case "Field":
    return join([ wrap("", print(r.alias), ": ") + print(r.name) + wrap("(", join(print(r.arguments), ", "), ")"), join(print(r.directives), " "), print(r.selectionSet) ], " ");

   case "StringValue":
    return r.isBlockString ? e(r.value) : n(r.value);

   case "BooleanValue":
    return r.value ? "true" : "false";

   case "NullValue":
    return "null";

   case "IntValue":
   case "FloatValue":
   case "EnumValue":
   case "Name":
    return r.value;

   case "ListValue":
    return "[" + join(print(r.values), ", ") + "]";

   case "ObjectValue":
    return "{" + join(print(r.fields), ", ") + "}";

   case "ObjectField":
    return r.name.value + ": " + print(r.value);

   case "Variable":
    return "$" + r.name.value;

   case "Document":
    return join(print(r.definitions), "\n\n") + "\n";

   case "SelectionSet":
    return block(print(r.selections));

   case "Argument":
    return r.name.value + ": " + print(r.value);

   case "FragmentSpread":
    return "..." + print(r.name) + wrap(" ", join(print(r.directives), " "));

   case "InlineFragment":
    return join([ "...", wrap("on ", print(r.typeCondition)), join(print(r.directives), " "), print(r.selectionSet) ], " ");

   case "FragmentDefinition":
    return "fragment " + r.name.value + wrap("(", join(print(r.variableDefinitions), ", "), ")") + " on " + print(r.typeCondition) + " " + wrap("", join(print(r.directives), " "), " ") + print(r.selectionSet);

   case "Directive":
    return "@" + r.name.value + wrap("(", join(print(r.arguments), ", "), ")");

   case "NamedType":
    return r.name.value;

   case "ListType":
    return "[" + print(r.type) + "]";

   case "NonNullType":
    return print(r.type) + "!";

   default:
    return "";
  }
}

var join = function(e, n) {
  return e && e.filter((function(e) {
    return e;
  })).join(n || "") || "";
};

var block = function(e) {
  return wrap("{\n  ", join(e, "\n").replace(/\n/g, "\n  "), "\n}");
};

var wrap = function(e, n, r) {
  return n ? e + n + (r || "") : "";
};

export { print };
