import { invariant as n } from "../jsutils/invariant.mjs";

import { print as e } from "../language/printer.mjs";

import { printBlockString as t } from "../language/blockString.mjs";

import { isIntrospectionType as r } from "../type/introspection.mjs";

import { DEFAULT_DEPRECATION_REASON as i, isSpecifiedDirective as p } from "../type/directives.mjs";

import { isScalarType as u, isObjectType as a, isInterfaceType as o, isUnionType as c, isEnumType as f, isInputObjectType as s } from "../type/definition.mjs";

import { astFromValue as m } from "./astFromValue.mjs";

function printIntrospectionSchema(t) {
  return function printFilteredSchema(t, r, i) {
    var p = t.getDirectives().filter(r);
    var m = Object.values(t.getTypeMap()).filter(i);
    return [ printSchemaDefinition(t) ].concat(p.map((function(n) {
      return function printDirective(n) {
        return printDescription(n) + "directive @" + n.name + printArgs(n.args) + (n.isRepeatable ? " repeatable" : "") + " on " + n.locations.join(" | ");
      }(n);
    })), m.map((function(t) {
      return function printType(t) {
        if (u(t)) {
          return function printScalar(n) {
            return printDescription(n) + "scalar " + n.name + function printSpecifiedByURL(n) {
              if (null == n.specifiedByURL) {
                return "";
              }
              return " @specifiedBy(url: " + e({
                kind: "StringValue",
                value: n.specifiedByURL
              }) + ")";
            }(n);
          }(t);
        }
        if (a(t)) {
          return function printObject(n) {
            return printDescription(n) + "type " + n.name + printImplementedInterfaces(n) + printFields(n);
          }(t);
        }
        if (o(t)) {
          return function printInterface(n) {
            return printDescription(n) + "interface " + n.name + printImplementedInterfaces(n) + printFields(n);
          }(t);
        }
        if (c(t)) {
          return function printUnion(n) {
            var e = n.getTypes();
            var t = e.length ? " = " + e.join(" | ") : "";
            return printDescription(n) + "union " + n.name + t;
          }(t);
        }
        if (f(t)) {
          return function printEnum(n) {
            var e = n.getValues().map((function(n, e) {
              return printDescription(n, "  ", !e) + "  " + n.name + printDeprecated(n.deprecationReason);
            }));
            return printDescription(n) + "enum " + n.name + printBlock(e);
          }(t);
        }
        if (s(t)) {
          return function printInputObject(n) {
            var e = Object.values(n.getFields()).map((function(n, e) {
              return printDescription(n, "  ", !e) + "  " + printInputValue(n);
            }));
            return printDescription(n) + "input " + n.name + printBlock(e);
          }(t);
        }
        n(!1);
      }(t);
    }))).filter(Boolean).join("\n\n");
  }(t, p, r);
}

function printSchemaDefinition(n) {
  if (null == n.description && function isSchemaOfCommonNames(n) {
    var e = n.getQueryType();
    if (e && "Query" !== e.name) {
      return !1;
    }
    var t = n.getMutationType();
    if (t && "Mutation" !== t.name) {
      return !1;
    }
    var r = n.getSubscriptionType();
    if (r && "Subscription" !== r.name) {
      return !1;
    }
    return !0;
  }(n)) {
    return;
  }
  var e = [];
  var t = n.getQueryType();
  if (t) {
    e.push("  query: " + t.name);
  }
  var r = n.getMutationType();
  if (r) {
    e.push("  mutation: " + r.name);
  }
  var i = n.getSubscriptionType();
  if (i) {
    e.push("  subscription: " + i.name);
  }
  return printDescription(n) + "schema {\n" + e.join("\n") + "\n}";
}

function printImplementedInterfaces(n) {
  var e = n.getInterfaces();
  return e.length ? " implements " + e.map((function(n) {
    return n.name;
  })).join(" & ") : "";
}

function printFields(n) {
  return printBlock(Object.values(n.getFields()).map((function(n, e) {
    return printDescription(n, "  ", !e) + "  " + n.name + printArgs(n.args, "  ") + ": " + String(n.type) + printDeprecated(n.deprecationReason);
  })));
}

function printBlock(n) {
  return 0 !== n.length ? " {\n" + n.join("\n") + "\n}" : "";
}

function printArgs(n, e) {
  if (void 0 === e) {
    e = "";
  }
  if (0 === n.length) {
    return "";
  }
  if (n.every((function(n) {
    return !n.description;
  }))) {
    return "(" + n.map(printInputValue).join(", ") + ")";
  }
  return "(\n" + n.map((function(n, t) {
    return printDescription(n, "  " + e, !t) + "  " + e + printInputValue(n);
  })).join("\n") + "\n" + e + ")";
}

function printInputValue(n) {
  var t = m(n.defaultValue, n.type);
  var r = n.name + ": " + String(n.type);
  if (t) {
    r += " = " + e(t);
  }
  return r + printDeprecated(n.deprecationReason);
}

function printDeprecated(n) {
  if (null == n) {
    return "";
  }
  if (n !== i) {
    return " @deprecated(reason: " + e({
      kind: "StringValue",
      value: n
    }) + ")";
  }
  return " @deprecated";
}

function printDescription(n, e, r) {
  if (void 0 === e) {
    e = "";
  }
  if (void 0 === r) {
    r = !0;
  }
  var i = n.description;
  if (null == i) {
    return "";
  }
  return (e && !r ? "\n" + e : e) + t(i).replace(/\n/g, "\n" + e) + "\n";
}

export { printIntrospectionSchema };
